package com.persagy.dmp.starter.alarm.communication.websocket;

import cn.hutool.core.collection.CollectionUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.util.Set;

/**
 * @description: websocket服务端
 * 由于是websocket 所以原本是@RestController的http形式直接替换成@ServerEndpoint即可，作用是一样的 就是指定一个地址表示定义一个websocket的Server端
 * @author: lixing
 * @company: Persagy Technology Co.,Ltd
 * @since: 2020/11/30 6:47 下午
 * @version: V1.0
 */
@ServerEndpoint(value = "/webSocketServer")
@Component
@Slf4j
public class AlarmWebSocketServer {

    @OnOpen
    public void onOpen(Session session) {
        log.info("有新的客户端建立连接，编号: " + session.getId());
        //将新用户存入在线的组
        AlarmWebSocketCache.addClient(session);
    }

    /**
     * 客户端关闭
     *
     * @param session session
     */
    @OnClose
    public void onClose(Session session) {
        String clientId = session.getId();
        log.info("客户端断开连接，编号:" + clientId);
        //将掉线的用户移除在线的组里
        AlarmWebSocketCache.removeClient(clientId);
    }

    /**
     * 发生错误
     *
     * @param throwable e
     */
    @OnError
    public void onError(Session session, Throwable throwable) {
        String id = "";
        if (null != session) {
            id = session.getId();
            AlarmWebSocketCache.removeClient(id);
        }
        log.info("客户端{}出错 ", id);
    }

    /**
     * 收到客户端发来消息
     *
     * @param message 消息对象，格式：point,org,1101010001,10001-101,10001-102,10001-103,10001-104
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        String clientId = session.getId();
        log.info("收到客户端{}的数据请求消息:{}", clientId, message);
        if ("closeSession".equals(message)) {
            onClose(session);
            return;
        }
    }


    /**
     * @param msg 消息
     * @description: 发送消息
     * @return: void
     * @exception:
     * @author: shiliqiang
     * @company: Persagy Technology Co.,Ltd
     * @since: 2020/10/21 22:30
     * @version: V1.0
     */
    public static void sendMsgToClients(String msg) throws Exception {
        Set<Session> clients = AlarmWebSocketCache.getClients();
        synchronized (clients) {
            if (CollectionUtil.isNotEmpty(clients)) {
                for (Session session : clients) {
                    if (null != session && session.isOpen()) {
                        //同步发送
                        session.getBasicRemote().sendText(msg);
                    }
                }
            }
        }
    }
}
